# SNMP::Info - test.pl
# $Id: test.pl,v 1.2 2003/02/19 17:38:30 maxbaker Exp $
#
# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'
#

use Test;
BEGIN { plan tests => 1 };

use SNMP::Info;
ok(1); 

# If we made it this far, we're ok.
#########################

# No tests yet.
print "No test's implemented yet. \n";
